﻿using UnityEngine;
using System.Collections;

/// <summary>
/// A soldier that moves along a hidden grid and fights its enemies. See BattlefieldGrid.cs.
/// Trigger colliders are what let me detect that an enemy is in melee range. If my action timer
/// reaches ~0 and an enemy is in range, then I'll attack.
/// (This might not be a good way to handle melee, since it assumes enemies are only present
/// in one direction. Better to rethink the attack AI as part of the process of allowing me to
/// turn around and chase enemies.)
/// </summary>



public class Troop : MonoBehaviour
{
	TroopGrid grid;
	public TroopArmy army;
	Vector3 target; // Exact coordinates I'll move to before acting again.
	bool is_moving = false;
	public int node_x; // Grid X
	public int node_z; // Grid Z

	public int faction; // I'll attack any other faction!

	public float action_wait_time = 0f;
	public float action_wait_time_max = 1f;

	public int health = 100;
	public int attack_power = 150;

	public ORDERS orders = ORDERS.HOLD;

	Animator animator;
	bool facing_left = true;

	public GameObject corpse;
	public GameObject projectile; // For ranged weapons.
	float projectile_angle = 0f;
	float projectile_speed = 2f;

	/// <summary>
	/// Set my grid location for AI purposes, as opposed to my Transform's coordinates.
	/// </summary>
	/// <param name="x">The x coordinate.</param>
	/// <param name="z">The z coordinate.</param>
	public void SetNode(int x, int z)
	{
		node_x = x;
		node_z = z;
	}

	void Start ()
	{
		grid = GameObject.Find("BattlefieldGrid").GetComponent<TroopGrid>();
		animator = GetComponent<Animator>();
		target = transform.position;

		if(faction == 0) { SetFacing(false); }
		else { SetFacing(true); }
	}

	void SetFacing(bool left)
	{
		if(facing_left != left) // If changed, flip the sprite.
		{
			Vector3 temp = transform.localScale;
            // In Unity 5 there's an explicit "flip sprite" option, but for 4 I'm doing it this way.
			temp.x *= -1;
			transform.localScale = temp;
		}
		facing_left = left;
		animator.SetBool("FacingLeft",left);
	}

	/// <summary>
	/// Get a number to add to my target X based on my facing. Eg. target_x += 1 * Facing();
	/// </summary>
	int Facing()
	{
		if(facing_left) return -1;
		else return 1;
	}

	void Update()
	{
		// Wait, if something has made me pause.
		if(action_wait_time > float.Epsilon)
		{
			action_wait_time -= Time.deltaTime;
			return;
		}
		action_wait_time = 0f;

		// Move.
		if(is_moving)
		{
			transform.position = Vector3.MoveTowards(transform.position,target,.25f);
			if(Vector3.Distance(transform.position,target) < float.Epsilon)
			{
				is_moving = false;
			}
		}
		else
		{
			// Pick a new destination.
			PickTarget();
		}
	}

	void PickTarget()
	{
		bool move_ok;
		int target_x;
		int target_z;

		switch(orders)
		{
			case ORDERS.HOLD:
				animator.SetBool("Moving",false);
				action_wait_time = action_wait_time_max;
				break;
			
			case ORDERS.ADVANCE:

				animator.SetBool("Moving",true);

				// Test: Sometimes randomly turn around.
				if(Random.Range(0,99) < 20)
				{
					SetFacing(!facing_left);
				}

				// Pick the node in front of me.
				target_x = node_x + 1 * Facing();
				if((target_x<0)||(target_x>=grid.GRID_SIZE_X)) break;
				target_z = node_z;

				// Randomly aim at the node diagonally "above" or "below" that one.
				if((node_z > 0) && (Random.value < .05f)) { target_z = node_z - 1; }
				else if((node_z < grid.GRID_SIZE_Z-1) && (Random.value < .05f)) { target_z = node_z + 1; }

				// Can I actually go there?
				move_ok = grid.IsNodeEmpty(target_x,target_z);
				//Debug.Log (name + ": Target: " + target_x + "," + target_z + ". Free: " + move_ok);
				if(move_ok)
				{
					target = new Vector3(grid.SPACING_X * target_x,
					                     1.5f,
					                     grid.SPACING_Z * target_z);
					grid.MoveSoldierToNode(this,target_x,target_z);
					is_moving = true;
					//Debug.Log(name + ": Moving to: " + target_x + ","+target_z);
				}
				break;

			case ORDERS.RANGED:
				if(action_wait_time > float.Epsilon) return;
				action_wait_time = action_wait_time_max * 3f;


				GameObject shot = (GameObject) Instantiate(projectile,transform.position,transform.rotation);
				shot.rigidbody.AddForce( new Vector3(600f + Random.Range(-50f,50f),600f + Random.Range(-50f,50f),0f) );
				shot.GetComponent<Projectile>().faction = faction;
				break;
			
			case ORDERS.CHARGE:
				/*
				TrackNearbySoldiers tns = GetComponentInChildren<TrackNearbySoldiers>();
				Debug.Log ("Found TNS (maybe): " + tns);
				if(!tns.SeeEnemies()) break;
				GameObject enemy = tns.GetEnemyToSeek();
				*/

				//WIP. Note that it's inefficient to seek a target every time we reach a node.
				Troop enemy_to_seek = army.enemy.GetComponent<TroopArmy>().FindAnEnemyToAttack(this,40f);
				// The target might be myself, which means no enemies are within range.
				if(enemy_to_seek == this) break; // No enemies within range.

				// Pick a node which is (in 8-way motion) towards the foe.
				// IN PROGRESS: Don't target the foe, but the place "in front of" them.
				/*
				target_x = node_x;
				target_z = node_z;
				if(enemy_to_seek.node_x > node_x) target_x += 1;
				else if(enemy_to_seek.node_x < node_x) target_x -= 1;
				if(enemy_to_seek.node_z > node_z) target_z += 1;
				else if(enemy_to_seek.node_z < node_z) target_z -= 1;
				*/
				target_x = node_x;
				target_z = node_z;

				// Kludge: seek adjacent node left/right, not target's actual space.
				if(enemy_to_seek.node_x > node_x) target_x -= 1;
				else target_x += 1;

				if(enemy_to_seek.node_x > node_x) target_x += 1;
				else if(enemy_to_seek.node_x < node_x) target_x -= 1;
				if(enemy_to_seek.node_z > node_z) target_z += 1;
				else if(enemy_to_seek.node_z < node_z) target_z -= 1;

				move_ok = grid.IsNodeEmpty(target_x,target_z);
				//Debug.Log (name + ": Target: " + target_x + "," + target_z + ". Free: " + move_ok);
				if(move_ok)
				{
					target = new Vector3(grid.SPACING_X * target_x,
					                     1.5f,
					                     grid.SPACING_Z * target_z);
					grid.MoveSoldierToNode(this,target_x,target_z);
					is_moving = true;
					//Debug.Log(name + ": Moving to: " + target_x + ","+target_z);
				}

				break;

			default:
				break;

		}
	}

	public void TakeDamage(int amount)
	{
		//Debug.Log (name + ": Taking damage!");
		health -= (int) (Random.value * amount);

		if(health<=0) // Die, leaving behind an auto-destroying sprite with a death animation.
		{
			grid.MarkNodeAsEmpty(node_x,node_z);
			GameObject my_corpse = (GameObject) Instantiate(corpse,transform.position,transform.rotation);

			if(!facing_left) // Flip the death-animation sprite.
			{
				Vector3 temp = my_corpse.transform.localScale;
				temp.x *= -1;
				my_corpse.transform.localScale = temp;
			}

			army.GetComponent<TroopArmy>().TellMeSoldierWasKilled(this); // Notify my unit thaat I'm gone.

			Destroy(gameObject); // Destroy self.
		}
	}
	
    // This function ought to get rewritten to handle attacking differently, but it works...
    // but shows an attack animation in whatever direction I'm facing regardless of enemy's location.
	void OnTriggerStay(Collider other)
	{
		// Another entity is within attack range.
		// If hostile, and haven't attacked lately, attack!
		
		if(other.tag != "Character") { return; }
		if((is_moving) || (action_wait_time > float.Epsilon)) { return; } // Need to wait before attacking.

		//Debug.Log (this.name + ": Sensed a character: " + other.name);

		Troop enemy_script = other.GetComponent<Troop>();
		//Debug.Log ("Encountered an enemy with faction: " + enemy_script.faction);
		if (enemy_script.faction != faction)
		{
			enemy_script.TakeDamage( (int) Random.Range(attack_power*.5f,attack_power));
			action_wait_time = action_wait_time_max + Random.value; // Start delay before I can attack again.
			animator.SetTrigger("Attacking");
		}
	}


}
