﻿using UnityEngine;
using System.Collections;

/// <summary>
/// A grid of int variables, representing which spots on an imaginary grid are occupied by who.
/// The CubeSoldiers' AI moves to a node on this grid, then picks another node to move to,
/// based on whether it's charging or what. A CubeSoldier can ask questions like "is the node
/// in front of me occupied by an enemy?" as part of its AI.
/// Movement is actually handled by Unity through ordinary GameObject/Transform coordinates.
/// It's just that a soldier will keep moving until it reaches (say) x10z5 before it decides
/// what to do next. It's important that a node never be occupied by two troops at once, so
/// any movement decision by a CubeSoldier must first make sure the destination node is not
/// occupied, and nothing else can be allowed to occupy the node until the occupant has moved
/// or died. This simplistic grid system avoids complex Unity pathfinding and allows for
/// hundreds of units at a reasonable framerate; I get 60 FPS on a decent laptop with 240 units.
/// </summary>


public class TroopGrid : MonoBehaviour
{
	int[,] nodes;
	
	public float SPACING_X = 3f;
	public float SPACING_Z = 2f;

	public int GRID_SIZE_X = 25;
	public int GRID_SIZE_Z = 12;

	public void Reset() { Application.LoadLevel("ManySoldiersWithoutNavMesh"); }

	void Awake()
	{
		nodes = new int[GRID_SIZE_X,GRID_SIZE_Z];
		for(int i=0;i<GRID_SIZE_X;i++)
		{
			for(int j=0;j<GRID_SIZE_Z;j++) { nodes[i,j] = 0; }
		}
	}

	/// <summary>
	/// Does this set of coordinates exist within range?
	/// </summary>
	/// <returns><c>true</c> if this location is within the allowed grid; otherwise, <c>false</c>.</returns>
	/// <param name="x">The x coordinate.</param>
	/// <param name="z">The z coordinate.</param>
	bool IsNodeValid(int x, int z)
	{
		return !((x<0) || (z<0) || (x>GRID_SIZE_X) || (z>GRID_SIZE_Z));
	}

	/// <summary>
	/// Sets soldier's grid location for AI purposes. (Soldier will move its Transform here before doing anything else.)
	/// </summary>
	/// <param name="troop">Troop.</param>
	/// <param name="x">The x coordinate.</param>
	/// <param name="z">The z coordinate.</param>
	public void MoveSoldierToNode(Troop troop,int x,int z)
	{
		if((x<0) || (z<0) || (x>GRID_SIZE_X) || (z>GRID_SIZE_Z)) // Could just use IsNodeValid.
		{
			Debug.Log("Invalid grid location: " + x + "," + z);
			return; // Invalid
		}

		MarkNodeAsEmpty(troop.node_x,troop.node_z);
		PutSoldierAtNode(troop,x,z);
	}

	/// <summary>
	/// Mark node as occupied by this faction, and tell troop it's here.
	/// </summary>
	/// <param name="troop">Troop.</param>
	/// <param name="x">The x coordinate.</param>
	/// <param name="z">The z coordinate.</param>
	public void PutSoldierAtNode(Troop troop, int x,int z)
	{
		//Debug.Log ("Troop: " + troop + "to " + x + "," + z);
		troop.SetNode(x,z);
		nodes[x,z] = troop.faction;
	}
	/// <summary>
	/// Mark this node as empty.
	/// </summary>
	/// <param name="x">The x coordinate.</param>
	/// <param name="z">The z coordinate.</param>
	public void MarkNodeAsEmpty(int x, int z)
	{
		if(!IsNodeValid(x,z)) return;
		nodes[x,z] = 0;
	}

	public bool IsNodeEmpty(int x, int z)
	{
		if(!IsNodeValid(x,z)) return false;
		return (nodes[x,z] == 0);
	}

	/// <summary>
	/// Says which faction occupies a node. Good for AI and a minimap.
	/// </summary>
	/// <returns>The faction at node.</returns>
	/// <param name="x">The x coordinate.</param>
	/// <param name="z">The z coordinate.</param>
	public int WhatFactionAtNode(int x, int z)
	{
		if(!IsNodeValid(x,z)) return 0;
		return nodes[x,z];
	}




}
